<?php

namespace EventPortal\Seeds;

use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Permission\Models\Role;
use EventPortal\Domain\User\Models\User;
use Illuminate\Database\Seeder;
use Illuminate\Support\Arr;

class AdminSeeder extends Seeder
{
    private $requireReset;

    public function __construct()
    {
        $this->requireReset = config('app.env') !== 'local';
    }

    public function run(): void
    {
        $this->createSuperAdmin();
        $this->createAdmin();
        $this->createTeam();
    }

    private function createSuperAdmin(): void
    {
        $role = Role::findByName('super-admin');

        $user = User::firstOrCreate([
            'email' => 'super-admin@example.com',
            'first_name' => 'Super',
            'last_name' => 'Admin',
        ], [
            'password' => bcrypt(config('auth.default_password')),
            'password_created_at' => $this->requireReset ? null : now(),
        ]);

        $admin = Admin::firstOrCreate([
            'username' => 'SA1234',
            'department' => 'Digital',
            'job_title' => 'Being',
            'phone' => '01234 56789',
        ]);

        $user->admin()->save($admin);

        $admin->assignRole($role);
    }

    private function createAdmin(): void
    {
        $role = Role::findByName('admin');

        $user = User::firstOrCreate([
            'email' => 'admin@example.com',
            'first_name' => 'Normal',
            'last_name' => 'Admin',
        ], [
            'password' => bcrypt(config('auth.default_password')),
            'password_created_at' => $this->requireReset ? null : now(),
        ]);

        $admin = Admin::firstOrCreate([
            'username' => 'S09876',
            'department' => 'Digital',
            'job_title' => 'Being',
            'phone' => '01234 23234',
        ]);

        $user->admin()->save($admin);

        $admin->assignRole($role);
    }

    private function createTeam(): void
    {
        $role = Role::findByName('super-admin');

        collect([
            [
                'first_name' => 'Lloyd',
                'last_name' => 'Culpepper',
                'department' => 'Digital',
                'job_title' => 'PHP Developer',
            ],
            [
                'first_name' => 'Lee',
                'last_name' => 'Pownall',
                'department' => 'Digital',
                'job_title' => 'Developer',
            ],
            [
                'first_name' => 'Joe',
                'last_name' => 'Priest',
                'department' => 'Digital',
                'job_title' => 'PHP Developer',
            ],
            [
                'first_name' => 'Ash',
                'last_name' => 'Packwood',
                'department' => 'Digital',
                'job_title' => 'PHP Developer',
            ],
            [
                'first_name' => 'Harsh',
                'last_name' => 'Bhatia',
                'department' => 'Digital',
                'job_title' => 'PHP Developer',
            ],
            [
                'first_name' => 'Dan',
                'last_name' => 'Jarrett',
                'department' => 'Digital',
                'job_title' => 'Front-End Developer',
            ],
            [
                'first_name' => 'Shaughn',
                'last_name' => 'Williams',
                'department' => 'Digital',
                'job_title' => 'Javascript Engineer',
            ],
            [
                'first_name' => 'Toby',
                'last_name' => 'Callinan',
                'department' => 'Digital',
                'job_title' => 'Developer',
            ],
            [
                'first_name' => 'Andy',
                'last_name' => 'Grierson',
                'department' => 'Digital',
                'job_title' => 'Senior Digital Project Manager',
            ],
        ])->each(function (array $member) use ($role): void {
            $name = Arr::only($member, ['first_name', 'last_name']);
            $email = strtolower(implode('.', $name)) . "@drpgroup.com";
            $user = User::updateOrCreate(
                ['email' => $email],
                array_merge(
                    $name,
                    [
                        'password' => bcrypt(config('auth.default_password')),
                        'password_created_at' => $this->requireReset ? null : now(),
                    ]
                )
            );

            $admin = Admin::updateOrCreate(
                ['user_id' => $user->id],
                Arr::only($member, ['department', 'job_title'])
            );

            $admin->assignRole($role);
        });
    }
}
